-- get the addon namespace
local addon, ns = ...
-- get the config values
local cfg = ns.cfg
-- holder for some lib functions
local lib = _G["HEDD_lib"] or CreateFrame("Frame","HEDD_lib")
ns.lib = lib

lib.CD_OnEvent = function(self, event, ...)
	if event~="SPELL_UPDATE_COOLDOWN" then return end
	local start, duration = GetSpellCooldown(self.spellID)
    if duration > 0 then
		CooldownFrame_SetTimer(self.cooldown,start, duration,1)
    else
        self.cooldown:Hide()
    end
end

lib.CD_OnEventItem = function(self, event, ...)
	oldtable=hedlib.shallowCopy(self)
	self.start, self.duration, self.enable = GetItemCooldown(self.itemID)
	if not hedlib.shallowCompare(self,oldtable) then return end
	if enable == 0 then
		hedlib.SetDesaturation(self.icon,1)
--		print("Disabled")
		return
	end
	hedlib.SetDesaturation(self.icon,nil)
    if self.duration > 0 then
		self.cooldown:Show()
        self.cooldown:SetCooldown(self.start, self.duration)
    else
        self.cooldown:Hide()
    end
end

lib.CDadd = function(spell,ids)
	if not cfg.spells[spell] then return end
	
	local btn = _G["HEDD_CD_"..spell] or CreateFrame("Button", "HEDD_CD_"..spell, Heddframe.CD, "SecureActionButtonTemplate,ActionButtonTemplate")
	if btn and not InCombatLockdown() then
		btn:Show()
	end
	btn.spellID=cfg.spells[spell].id
	btn:SetWidth(cfg.isize)
	btn:SetHeight(cfg.isize)
	btn:SetAttribute("*type1", "spell")
	btn:SetAttribute("spell", cfg.spells[spell].name)
	btn:RegisterForClicks("AnyUp")
	btn.icon:SetTexture(cfg.spells[spell].icon)
	btn:ClearAllPoints()
	if not cfg.cd_num then
		btn:SetPoint("TOP",Heddframe.CD,"BOTTOM")
		cfg.cd_num=1
	else
		cfg.cd_num=cfg.cd_num+1
		btn:SetPoint("LEFT",("HEDD_CD_"..cfg.cd_last),"RIGHT",10,0)
	end
	cfg.cd_last=spell

	local cd = _G["HEDD_CD_"..spell.."Cooldown"] or CreateFrame("Cooldown", "$parentCooldown", btn, "CooldownFrameTemplate")
	cd:SetAllPoints(true)
	cd:SetFrameLevel(4)
	cd:Hide()
	btn.cooldown=cd

	local shine = _G["HEDD_CD_"..spell.."Shine"] or CreateFrame("FRAME", "$parentShine", btn, "SpellBookShineTemplate") --"SpellBookShineTemplate,AutoCastShineTemplate")
	shine:SetFrameStrata("MEDIUM")
	shine:SetFrameLevel(5)
	shine:SetAllPoints(true)
	btn.shine=shine
	AutoCastShine_AutoCastStart(btn.shine)
	
	if HeddDB[btn.spellID] then btn.shine:Hide()
	else btn.shine:Show() end
	--btn:Show()
	btn:RegisterEvent("SPELL_UPDATE_COOLDOWN")
	btn:SetScript("OnEvent", lib.CD_OnEvent)
	lib.CD_OnEvent(btn)
	btn:SetScript("PostClick", function(self, button)
	if button~="LeftButton" then 
		lib.CDtoggle(spell)
	end
	end)
	if ids then btn.ids = ids end
	btn.timers={}
end

lib.CDaddTimers = function(spell,show,func,events,noflash)
	local btn = _G["HEDD_CD_"..spell]
	if btn then
		local timer=#btn.timers+1
		btn.timers[timer]={}
		btn.timers[timer].frame=_G["HEDD_CD_"..spell.."_Timer"..timer] or CreateFrame("Frame","HEDD_CD_"..spell.."_Timer"..timer,btn)
		btn.timers[timer].frame:SetFrameStrata("MEDIUM")
		btn.timers[timer].frame:SetFrameLevel(9+timer)
		btn.timers[timer].frame:SetAllPoints(true)
		btn.timers[timer].frame.btn=btn
		btn.timers[timer].frame.cooldown = _G["HEDD_CD_"..spell.."_Timer"..timer.."_Cooldown"] or CreateFrame("Cooldown", "$parent_Cooldown", btn.timers[timer].frame, "CooldownFrameTemplate")
		btn.timers[timer].frame.cooldown:SetAllPoints(true)
		btn.timers[timer].frame.cooldown:Hide()
		btn.timers[timer].frame.cooldown.icon=_G["HEDD_CD_"..spell.."_Timer"..timer.."_Texture"] or btn.timers[timer].frame:CreateTexture("HEDD_CD_"..spell.."_Timer"..timer.."_Texture","BACKGROUND")
		btn.timers[timer].frame.cooldown.icon:SetAllPoints(btn.timers[timer].frame)
		btn.timers[timer].frame.cooldown.icon:SetTexture(lib.GetSpellIcon(show))
		btn.timers[timer].frame.cooldown.icon:Hide()
		hedlib.CreateBD(btn.timers[timer].frame,cfg.cdborder,{1,0,0,1})
--		btn.timers[timer].frame.bd:SetFrameStrata("MEDIUM")
--		btn.timers[timer].frame.bd:SetFrameLevel(100)
		btn.timers[timer].frame.bd:Hide()
		btn.timers[timer].frame.cooldown:HookScript("OnShow", function(self)
		self.icon:Show()
		self:GetParent().bd:Show()
		self:GetParent().btn.icon:Hide()
		
		--[[if not noflash then
			UIFrameFlash(self:GetParent(), 0.5, 0.5, -1,true)
		end]]
		end)
		btn.timers[timer].frame.cooldown:HookScript("OnHide", function(self)
		self.icon:Hide()
		self:GetParent().bd:Hide()
		self:GetParent().btn.icon:Show()
		--[[if not noflash then
			UIFrameFlashStop(self:GetParent())
		end]]
		end)
		if func then
			btn.timers[timer].frame:SetScript("OnEvent", func)
		end
		if events then
			for _, value in ipairs(events) do
				btn.timers[timer].frame:RegisterEvent(value)
			end
		end
	end
end

lib.CDaddStacks = function(spell,func,events)
	local btn = _G["HEDD_CD_"..spell]
	if btn then
		btn.text = hedlib.CreateFont(btn, cfg.textfont, cfg.tsize-2, "OUTLINE")
		btn.text:SetPoint("BOTTOMRIGHT",btn,"BOTTOMRIGHT")
		btn.text:SetText("")
		if func then
			btn:HookScript("OnEvent", func)
		end
		if events then
			for _, value in ipairs(events) do
				btn:RegisterEvent(value)
			end
		end
	end
end


lib.Itemadd = function(item)
	if not item then return nil end
	local texture=select(10,GetItemInfo(item))
	if not texture then return nil end
	
	local btn = _G["HEDD_CD_"..item] or CreateFrame("Button", "HEDD_CD_"..item, Heddframe.CD, "SecureActionButtonTemplate,ActionButtonTemplate")
	btn.itemID=item
	btn:SetWidth(cfg.isize)
	btn:SetHeight(cfg.isize)
	btn:SetAttribute("*type1", "item")
	btn:SetAttribute("item", "item:"..item)
	btn:RegisterForClicks("AnyUp")
	btn.icon:SetTexture(texture)
	btn:ClearAllPoints()
	if not cfg.cd_num then
		btn:SetPoint("TOP",Heddframe.CD,"BOTTOM")
		cfg.cd_num=1
	else
		cfg.cd_num=cfg.cd_num+1
		btn:SetPoint("LEFT",("HEDD_CD_"..cfg.cd_last),"RIGHT",10,0)
	end
	cfg.cd_last=item
	
	
	local cd = _G["HEDD_CD_"..item.."Cooldown"] or CreateFrame("Cooldown", "$parentCooldown", btn, "CooldownFrameTemplate")
	cd:SetAllPoints(true)
	cd:SetFrameLevel(4)
	cd:Hide()
	btn.cooldown=cd
	btn:Show()
	btn:RegisterEvent("SPELL_UPDATE_COOLDOWN")
	btn:RegisterEvent("PLAYER_REGEN_ENABLED")
	btn:SetScript("OnEvent", lib.CD_OnEventItem)
	lib.CD_OnEventItem(btn)
end

lib.CDturnoff = function(spell)
	if not cfg.spells[spell] then return end
	local btn = _G["HEDD_CD_"..spell]
	if btn then
		btn.shine:Hide()
		HeddDB[btn.spellID]=true
		if btn.ids then
			for _, value in ipairs(btn.ids) do
				HeddDB[value]=true
			end
		end
		cfg.Update=true
		btn:SetScript("PostClick",nil)
	end
end

lib.CDtoggleOn = function(spell)
	if not cfg.spells[spell] then return end
	local btn = _G["HEDD_CD_"..spell]
	if btn then
		btn.shine:Show()
		HeddDB[btn.spellID]=false
		if btn.ids then
			for _, value in ipairs(btn.ids) do
				HeddDB[value]=false
			end
		end
		cfg.Update=true
		btn:SetScript("PostClick", function(self, button)
			if button~="LeftButton" then 
				lib.CDtoggle(spell)
			end
		end)
	end
end

lib.CDtoggleOff = function(spell)
	if not cfg.spells[spell] then return end
	local btn = _G["HEDD_CD_"..spell]
	if btn then
		btn.shine:Hide()
		HeddDB[btn.spellID]=true
		if btn.ids then
			for _, value in ipairs(btn.ids) do
				HeddDB[value]=true
			end
		end
		cfg.Update=true
		btn:SetScript("PostClick", function(self, button)
			if button~="LeftButton" then 
				lib.CDtoggle(spell)
			end
		end)
	end
end

lib.CDtoggle = function(spell)
	if not cfg.spells[spell] then return end
	local btn = _G["HEDD_CD_"..spell]
	if btn then
		if HeddDB[btn.spellID] then
			btn.shine:Show()
			HeddDB[btn.spellID]=false
			if btn.ids then
			for _, value in ipairs(btn.ids) do
				HeddDB[value]=false
			end
		end
			cfg.Update=true
			--print(spell.." show")
		else
			btn.shine:Hide()
			HeddDB[btn.spellID]=true
			if btn.ids then
			for _, value in ipairs(btn.ids) do
				HeddDB[value]=true
			end
		end
			cfg.Update=true
			--print(spell.." hide")
		end
	end
end

lib.CDdel = function(spell)
	local btn=_G["HEDD_CD_"..spell]
	if btn and not InCombatLockdown() then
		btn:Hide()
	end
end

lib.CDresize = function(spell)
	local btn=_G["HEDD_CD_"..spell]
	if btn then
		btn:SetWidth(cfg.isize)
		btn:SetHeight(cfg.isize)
	end
end